# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2012 Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
#   Author: Dave Malcolm <dmalcolm@redhat.com>
#
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Screenscraper for the output of "ifconfig"
# The relevant sources for ifconfig can be found in e.g.:
#   net-tools-1.60/lib/interface.c
# within ife_print_long()

import os
import re
from subprocess import Popen, PIPE
import unittest

__all__ = ['IfConfig']

# any whitespace:
ws = '\s+'

# any non-whitespace, as a delimited group:
group_nonws = '(\S+)'

# a decimal number, as a delimited group:
group_dec = '([0-9]+)'

# hexadecimal number of the form "0xf2600000"
group_o_hex = '0x([0-9a-f]+)'

# hexadecimal number without the "0x" prefix, e.g. "f2600000"
group_hex = '([0-9a-f]+)'

dot = r'\.'

dec = '[0-9]+'

group_ip4addr = ('(' + dec + dot +
                 dec + dot +
                 dec + dot +
                 dec + ')')


def parse_ip4addr(addr):
    import socket
    return (socket.inet_aton(addr))


class IfConfig:
    """
    Wrapper around a single invocation of "ifconfig"
    """

    def __init__(self, stdout=None, debug=False):
        if stdout is not None:
            self.stdout = stdout
            self.stderr = ''
        else:
            env = os.environ.copy()
            env.update(LANG='C.utf8')
            p = Popen('ifconfig', stdout=PIPE, stderr=PIPE,
                      env=env, universal_newlines=True)
            self.stdout, self.stderr = p.communicate()
            if self.stderr != '':
                raise ValueError(
                    'stderr from ifconfig was nonempty:\n%s' % self.stderr)
        if 0:
            print('self.stdout: %r' % self.stdout)
        self.devices = []
        curdev = None
        for line in self.stdout.splitlines():
            if 0:
                print('line: %r' % line)

            if line == '':
                continue

            # Start of a device entry, e.g:
            #  'lo: flags=73<UP,LOOPBACK,RUNNING>  mtu 16436'
            # old ifconfig format:
            #  'lo        Link encap:Local Loopback'
            m = re.match(r'(\w+): (.+)', line)
            mo = re.match(r'(\w+)\s+(.+)', line)
            if m:
                self.oldFormat = False
                devname = m.group(1)
                curdev = Device(devname, debug)
                self.devices.append(curdev)
                curdev._parse_rest_of_first_line(m.group(2))
                continue

            if mo:
                self.oldFormat = True
                devname = mo.group(1)
                curdev = Device(devname, debug)
                self.devices.append(curdev)
                curdev._parse_rest_of_first_line_old(mo.group(2))
                continue

            # If we don't have current device yet, doesn't make sense to
            # read the rest of output lines
            if curdev is None:
                continue

            if self.oldFormat:
                curdev._parse_line_old(line)
            else:
                curdev._parse_line(line)

    def get_device_by_name(self, devname):
        for dev in self.devices:
            if dev.name == devname:
                return dev
        raise ValueError('device not found: %r' % devname)


class Device:
    """
    Wrapper around a device entry within the output of "ifconfig"
    """

    def __init__(self, name, debug=False):
        self.name = name
        self.debug = debug

        self.flagsint = None
        self.flagsstr = None
        self.mtu = None
        self.metric = None
        self.outfill = None
        self.keepalive = None

        self.inet = None
        self.netmask = None
        self.broadcast = None
        self.destination = None

        self.inet6 = None
        self.prefixlen = None
        self.scopeid = None

        self.hwname = None
        self.hwaddr = None
        self.txqueuelen = None
        self.hwtitle = None

        self.rxpackets = None
        self.rxbytes = None

        self.rxerrors = None
        self.rxdropped = None
        self.rxoverruns = None
        self.rxframe = None

        self.rxcompressed = None

        self.txpackets = None
        self.txbytes = None

        self.txerrors = None
        self.txdropped = None
        self.txoverruns = None
        self.txcarrier = None
        self.txcollisions = None

        self.txcompressed = None

        self.interrupt = None
        self.baseaddr = None
        self.memstart = None
        self.memend = None
        self.dma = None

    def get_netmask_bits(self):
        # Convert a dotted netmask string to a bitcount int
        # e.g. from "255.255.252.0" to 22:
        if not self.netmask:
            return 0
        packed = parse_ip4addr(self.netmask)
        # count bits in "packed":
        result = 0
        for ch in packed:
            ch = ord(ch) if isinstance(ch, str) else ch
            while ch:
                if ch & 1:
                    result += 1
                ch //= 2
        return result

    def __repr__(self):
        return ('Device(name=%(name)r, flagsint=%(flagsint)r, '
                'flagsstr=%(flagsstr)r, mtu=%(mtu)r)'
                % (self.__dict__))

    def _debug(self, groups):
        if self.debug:
            print('groups: %r' % (groups,))

    def _parse_rest_of_first_line(self, text):
        m = re.match(r'flags=([0-9]+)<(.*)>  mtu ([0-9]+)(.*)', text)
        if not m:
            raise ValueError('unable to parse: %r' % text)
        self._debug(m.groups())
        self.flagsint = int(m.group(1))
        self.flagsstr = m.group(2)
        self.mtu = int(m.group(3))

        # It might have "  outfill %d  keepalive %d":
        m = re.match(ws
                     + 'outfill' + ws + group_dec + ws
                     + 'keepalive' + ws + group_dec,
                     m.group(4))
        if m:
            self._debug(m.groups())
            self.outfill = int(m.group(1))
            self.keepalive = int(m.group(2))

    def _parse_rest_of_first_line_old(self, text):
        m = re.match('Link encap:(\w+ ?\w*)\s*(HWaddr )?(\S*)', text)
        if not m:
            raise ValueError('unable to parse: %r' % text)
        self.hwtitle = m.group(1).strip()
        if m.group(2):
            self.hwaddr = m.group(3)

    def _parse_line(self, line):
        m = re.match(ws
                     + 'inet ' + group_ip4addr + ws
                     + 'netmask ' + group_ip4addr + '(.*)',
                     line)
        if m:
            self._debug(m.groups())
            self.inet = m.group(1)
            self.netmask = m.group(2)
            # "broadcast" and "destination" are both optional:
            line = m.group(3)
            m = re.match(ws + 'broadcast ' + group_ip4addr + '(.*)', line)
            if m:
                self.broadcast = m.group(1)
                line = m.group(2)

            m = re.match(ws + 'destination ' + group_nonws, line)
            if m:
                self.destination = m.group(1)
            return

        m = re.match(ws
                     + 'inet6 ' + group_nonws + ws
                     + 'prefixlen' + ws + group_dec + ws
                     + 'scopeid ' + group_nonws,
                     line)
        if m:
            self._debug(m.groups())
            self.inet6 = m.group(1)
            self.prefixlen = int(m.group(2))
            self.scopeid = m.group(3)
            return

        # e.g. (with hwaddr):
        #  '        ether ff:00:11:22:33:42  txqueuelen 1000  (Ethernet)'
        m = re.match(ws
                     + group_nonws + ws + group_nonws + ws
                     + 'txqueuelen' + ws + group_dec + ws
                     + '\((.*)\)', line)
        if m:
            self._debug(m.groups())
            self.hwname = m.group(1)
            self.hwaddr = m.group(2)
            self.txqueuelen = int(m.group(3))
            self.hwtitle = m.group(4)
            return

        # e.g. (without hwaddr):
        #  '        loop  txqueuelen 0  (Local Loopback)'
        m = re.match(ws
                     + group_nonws + ws
                     + 'txqueuelen' + ws + group_dec + ws
                     + '\((.*)\)', line)
        if m:
            self._debug(m.groups())
            self.hwname = m.group(1)
            self.hwaddr = None
            self.txqueuelen = int(m.group(2))
            self.hwtitle = m.group(3)
            return

        # e.g. '        RX packets 5313261  bytes 6062336615 (5.6 GiB)'
        m = re.match(ws
                     + 'RX packets ' + group_dec + ws
                     + 'bytes ' + group_dec + ws
                     + '\(.*\)',
                     line)
        if m:
            self._debug(m.groups())
            self.rxpackets = int(m.group(1))
            self.rxbytes = int(m.group(2))
            return

        # e.g. '        RX errors 0  dropped 0  overruns 0  frame 0'
        m = re.match(ws
                     + 'RX errors ' + group_dec + ws
                     + 'dropped ' + group_dec + ws
                     + 'overruns ' + group_dec + ws
                     + 'frame ' + group_dec,
                     line)
        if m:
            self._debug(m.groups())
            self.rxerrors = int(m.group(1))
            self.rxdropped = int(m.group(2))
            self.rxoverruns = int(m.group(3))
            self.rxframe = int(m.group(4))
            return

        # e.g. '        TX packets 2949524  bytes 344092625 (328.1 MiB)'
        m = re.match(ws
                     + 'TX packets ' + group_dec + ws
                     + 'bytes ' + group_dec + ws
                     + '\(.*\)',
                     line)
        if m:
            self._debug(m.groups())
            self.txpackets = int(m.group(1))
            self.txbytes = int(m.group(2))
            return

        # e.g. '        TX errors 0  dropped 0 overruns 0  carrier 0
        # collisions 0'
        m = re.match(ws
                     + 'TX errors ' + group_dec + ws
                     + 'dropped ' + group_dec + ws
                     + 'overruns ' + group_dec + ws
                     + 'carrier ' + group_dec + ws
                     + 'collisions ' + group_dec,
                     line)
        if m:
            self._debug(m.groups())
            self.txerrors = int(m.group(1))
            self.txdropped = int(m.group(2))
            self.txoverruns = int(m.group(3))
            self.txcarrier = int(m.group(4))
            self.txcollisions = int(m.group(5))
            return

        # e.g.'        device interrupt 20  memory 0xf2600000-f2620000  '
        m = re.match(ws
                     + 'device' + ws + '(.*)',
                     line)
        if m:
            self._debug(m.groups())
            line = m.group(1)
            m = re.match('interrupt ' + group_dec + ws + '(.*)',
                         line)
            if m:
                self.interrupt = int(m.group(1))
                line = m.group(2)

            m = re.match('base ' + group_o_hex + ws + '(.*)',
                         line)
            if m:
                self.baseaddr = int(m.group(1), 16)
                line = m.group(2)

            m = re.match('memory ' + group_o_hex + '-' +
                         group_hex + ws + '(.*)',
                         line)
            if m:
                self.memstart = int(m.group(1), 16)
                self.memend = int(m.group(2), 16)
                line = m.group(3)

            m = re.match('dma ' + group_o_hex,
                         line)
            if m:
                self.dma = int(m.group(1), 16)

            return

        raise ValueError('parser could not handle line: %r' % line)

    def _parse_line_old(self, line):
        m = re.match(ws
                     + 'inet addr:' + group_ip4addr
                     + '(.*)',
                     line)
        if m:
            self._debug(m.groups())
            self.inet = m.group(1)
            # "destination (P-t-P)" and "Bcast" are both optional:
            line = m.group(2)
            m = re.match(ws + 'P-t-P:' + group_ip4addr + '(.*)', line)
            if m:
                self.destination = m.group(1)
                line = m.group(2)

            m = re.match(ws + 'Bcast:' + group_ip4addr + '(.*)', line)
            if m:
                self.broadcast = m.group(1)
                line = m.group(2)

            m = re.match(ws + 'Mask:' + group_ip4addr, line)
            if m:
                self.netmask = m.group(1)
            return

        m = re.match(ws
                     + 'inet6 addr: ' + group_nonws + '/'
                     + group_dec + ws
                     + 'Scope:' + group_nonws,
                     line)
        if m:
            self._debug(m.groups())
            self.inet6 = m.group(1)
            self.prefixlen = int(m.group(2))
            self.scopeid = m.group(3)
            return

        # UP BROADCAST RUNNING MULTICAST  MTU:1500  Metric:1
        m = re.match(ws + '(.*)'
                     + '  MTU:' + group_dec + ws
                     + 'Metric:' + group_dec
                     + '(.*)', line)
        if m:
            self._debug(m.groups())
            self.flagsstr = m.group(1)
            self.mtu = int(m.group(2))
            self.metric = int(m.group(3))

            # It might have "  Outfill:%d  Keepalive:%d":
            m = re.match(ws
                         + 'Outfill:' + group_dec + ws
                         + 'Keepalive:' + group_dec,
                         m.group(4))
            if m:
                self._debug(m.groups())
                self.outfill = int(m.group(1))
                self.keepalive = int(m.group(2))
            return

        # RX packets:4458926 errors:0 dropped:0 overruns:0 frame:0
        m = re.match(ws
                     + 'RX packets:' + group_dec + ws
                     + 'errors:' + group_dec + ws
                     + 'dropped:' + group_dec + ws
                     + 'overruns:' + group_dec + ws
                     + 'frame:' + group_dec,
                     line)
        if m:
            self._debug(m.groups())
            self.rxpackets = int(m.group(1))
            self.rxerrors = int(m.group(2))
            self.rxdropped = int(m.group(3))
            self.rxoverruns = int(m.group(4))
            self.rxframe = int(m.group(5))
            return

        # "             compressed:%lu\n"
        m = re.match(ws + 'compressed:' + group_dec, line)
        if m:
            self.rxcompressed = int(m.group(1))
            return

        # TX packets:3536982 errors:0 dropped:0 overruns:0 carrier:0
        m = re.match(ws
                     + 'TX packets:' + group_dec + ws
                     + 'errors:' + group_dec + ws
                     + 'dropped:' + group_dec + ws
                     + 'overruns:' + group_dec + ws
                     + 'carrier:' + group_dec,
                     line)
        if m:
            self._debug(m.groups())
            self.txpackets = int(m.group(1))
            self.txerrors = int(m.group(2))
            self.txdropped = int(m.group(3))
            self.txoverruns = int(m.group(4))
            self.txcarrier = int(m.group(5))
            return

        # "          collisions:%lu compressed:%lu txqueuelen:%d "
        m = re.match(ws + 'collisions:' + group_dec + ws + '(.*)', line)
        if m:
            self._debug(m.groups())
            self.txcollisions = int(m.group(1))
            line = m.group(2)
            m = re.match('compressed:' + group_dec + ws + '(.*)', line)
            if m:
                self.txcompressed = int(m.group(1))
                line = m.group(2)

            m = re.match('txqueuelen:' + group_dec, line)
            if m:
                self.txqueuelen = int(m.group(1))
            return

        # RX bytes:3380060233 (3.1 GiB)  TX bytes:713438255 (680.3 MiB)
        m = re.match(ws + 'RX bytes:' + group_dec + ws + '\(.*\)' + ws
                     + 'TX bytes:' + group_dec + ws + '\(.*\)',
                     line)
        if m:
            self._debug(m.groups())
            self.rxbytes = int(m.group(1))
            self.txbytes = int(m.group(2))
            return

        # Interrupt:17 Memory:da000000-da012800
        m = re.match(ws + 'Interrupt:' + group_dec + '\s*(.*)', line)
        if m:
            self._debug(m.groups())
            self.interrupt = int(m.group(1))
            line = m.group(2)

        m = re.match('Base address:' + group_o_hex + '\s*(.*)', line)
        if m:
            self._debug(m.groups())
            self.baseaddr = int(m.group(1), 16)
            line = m.group(2)

        m = re.match('Memory:' + group_hex + '-' + group_hex + '\s*(.*)', line)
        if m:
            self._debug(m.groups())
            self.memstart = int(m.group(1), 16)
            self.memend = int(m.group(2), 16)
            line = m.group(3)

        m = re.match('DMA chan:' + group_hex, line)
        if m:
            self._debug(m.groups())
            self.dma = int(m.group(1), 16)

        return

# ifconfig = IfConfig()
# for dev in ifconfig.devices:
#    print(dev)


class ParserTests(unittest.TestCase):
    def test_full(self):
        # Parse a complex pre-canned output from ifconfig
        # This is the output of "ifconfig" on this machine, sanitized
        # to remove stuff that identifies it
        full = '''
eth1: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
        inet 1.12.123.124  netmask 255.255.252.0  broadcast 1.12.123.255
        inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
        ether ff:00:11:22:33:42  txqueuelen 1000  (Ethernet)
        RX packets 5329294  bytes 6074440831 (5.6 GiB)
        RX errors 0  dropped 0  overruns 0  frame 0
        TX packets 2955574  bytes 344607935 (328.6 MiB)
        TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
''' + '        device interrupt 20  memory 0xf2600000-f2620000  ' + '''

lo: flags=73<UP,LOOPBACK,RUNNING>  mtu 16436
        inet 127.0.0.1  netmask 255.0.0.0
        inet6 ::1  prefixlen 128  scopeid 0x10<host>
        loop  txqueuelen 0  (Local Loopback)
        RX packets 1562620  bytes 524530977 (500.2 MiB)
        RX errors 0  dropped 0  overruns 0  frame 0
        TX packets 1562620  bytes 524530977 (500.2 MiB)
        TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0

virbr0: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
        inet 1.12.123.124  netmask 255.255.252.0  broadcast 1.12.123.255
        ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
        RX packets 88730  bytes 5140566 (4.9 MiB)
        RX errors 0  dropped 0  overruns 0  frame 0
        TX packets 177583  bytes 244647206 (233.3 MiB)
        TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0

vnet0: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
        inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
        ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
        RX packets 538  bytes 172743 (168.6 KiB)
        RX errors 0  dropped 0  overruns 0  frame 0
        TX packets 26593  bytes 1379054 (1.3 MiB)
        TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0

vnet1: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
        inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
        ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
        RX packets 71567  bytes 5033151 (4.7 MiB)
        RX errors 0  dropped 0  overruns 0  frame 0
        TX packets 216553  bytes 200424748 (191.1 MiB)
        TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
'''
        ifconfig = IfConfig(stdout=full)
        self.assertEqual(len(ifconfig.devices), 5)

        # Verify eth1:
        eth1 = ifconfig.devices[0]
        # eth1: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
        self.assertEqual(eth1.name, 'eth1')
        self.assertEqual(eth1.flagsint, 4163)
        self.assertEqual(eth1.flagsstr, 'UP,BROADCAST,RUNNING,MULTICAST')
        self.assertEqual(eth1.mtu, 1500)
        # inet 1.12.123.124  netmask 255.255.252.0  broadcast 1.12.123.255
        self.assertEqual(eth1.inet, '1.12.123.124')
        self.assertEqual(eth1.netmask, '255.255.252.0')
        self.assertEqual(eth1.broadcast, '1.12.123.255')
        # inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
        self.assertEqual(eth1.inet6, 'ffff::ffff:ffff:ffff:ffff')
        self.assertEqual(eth1.prefixlen, 64)
        self.assertEqual(eth1.scopeid, '0x20<link>')
        #        ether ff:00:11:22:33:42  txqueuelen 1000  (Ethernet)
        self.assertEqual(eth1.hwname, 'ether')
        self.assertEqual(eth1.hwaddr, 'ff:00:11:22:33:42')
        self.assertEqual(eth1.txqueuelen, 1000)
        self.assertEqual(eth1.hwtitle, 'Ethernet')
        #        RX packets 5329294  bytes 6074440831 (5.6 GiB)
        self.assertEqual(eth1.rxpackets, 5329294)
        self.assertEqual(eth1.rxbytes, 6074440831)
        #        RX errors 0  dropped 0  overruns 0  frame 0
        self.assertEqual(eth1.rxerrors, 0)
        self.assertEqual(eth1.rxdropped, 0)
        self.assertEqual(eth1.rxoverruns, 0)
        self.assertEqual(eth1.rxframe, 0)
        #        TX packets 2955574  bytes 344607935 (328.6 MiB)
        self.assertEqual(eth1.txpackets, 2955574)
        self.assertEqual(eth1.txbytes, 344607935)
        #        TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
        self.assertEqual(eth1.txerrors, 0)
        self.assertEqual(eth1.txdropped, 0)
        self.assertEqual(eth1.txoverruns, 0)
        self.assertEqual(eth1.txcarrier, 0)
        self.assertEqual(eth1.txcollisions, 0)
        #        device interrupt 20  memory 0xf2600000-f2620000
        self.assertEqual(eth1.interrupt, 20)

        # Verify lo:
        lo = ifconfig.devices[1]
        self.assertEqual(lo.name, 'lo')
        # lo: flags=73<UP,LOOPBACK,RUNNING>  mtu 16436
        self.assertEqual(lo.flagsint, 73)
        self.assertEqual(lo.flagsstr, 'UP,LOOPBACK,RUNNING')
        self.assertEqual(lo.mtu, 16436)
        #         inet 127.0.0.1  netmask 255.0.0.0
        self.assertEqual(lo.inet, '127.0.0.1')
        self.assertEqual(lo.netmask, '255.0.0.0')
        self.assertEqual(lo.broadcast, None)
        #         inet6 ::1  prefixlen 128  scopeid 0x10<host>
        self.assertEqual(lo.inet6, '::1')
        self.assertEqual(lo.prefixlen, 128)
        self.assertEqual(lo.scopeid, '0x10<host>')
        #         loop  txqueuelen 0  (Local Loopback)
        self.assertEqual(lo.hwname, 'loop')
        self.assertEqual(lo.txqueuelen, 0)
        self.assertEqual(lo.hwtitle, 'Local Loopback')
        #         RX packets 1562620  bytes 524530977 (500.2 MiB)
        #         RX errors 0  dropped 0  overruns 0  frame 0
        #         TX packets 1562620  bytes 524530977 (500.2 MiB)
        #         TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0

        # Verify virbr0:
        virbr0 = ifconfig.devices[2]
        self.assertEqual(virbr0.name, 'virbr0')
        # virbr0: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
        #  inet 1.12.123.124  netmask 255.255.252.0  broadcast 1.12.123.255
        #  ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
        #  RX packets 88730  bytes 5140566 (4.9 MiB)
        #  RX errors 0  dropped 0  overruns 0  frame 0
        #  TX packets 177583  bytes 244647206 (233.3 MiB)
        #  TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0

        # Verify vnet0:
        vnet0 = ifconfig.devices[3]
        self.assertEqual(vnet0.name, 'vnet0')
        # vnet0: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
        #  inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
        #  ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
        #  RX packets 538  bytes 172743 (168.6 KiB)
        #  RX errors 0  dropped 0  overruns 0  frame 0
        #  TX packets 26593  bytes 1379054 (1.3 MiB)
        #  TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0

        # Verify vnet1:
        vnet1 = ifconfig.devices[4]
        self.assertEqual(vnet1.name, 'vnet1')
        # vnet1: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500
        #  inet6 ffff::ffff:ffff:ffff:ffff  prefixlen 64  scopeid 0x20<link>
        #  ether ff:00:11:22:33:42  txqueuelen 0  (Ethernet)
        #  RX packets 71567  bytes 5033151 (4.7 MiB)
        #  RX errors 0  dropped 0  overruns 0  frame 0
        #  TX packets 216553  bytes 200424748 (191.1 MiB)
        #  TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0

        fullOld = '''
eth0      Link encap:Ethernet  HWaddr 00:11:22:33:44:55
          inet addr:1.12.123.124  Bcast:1.12.123.255  Mask:255.255.252.0
          inet6 addr: dddd::dddd:dddd:dddd:dddd:dddd:dddd/64 Scope:Site
          inet6 addr: eeee::eeee:eeee:eeee:eeee:eeee:eeee/64 Scope:Global
          inet6 addr: ffff::ffff:ffff:ffff:ffff/64 Scope:Link
          UP BROADCAST RUNNING MULTICAST  MTU:1500  Metric:1
          RX packets:4458926 errors:0 dropped:0 overruns:0 frame:0
          TX packets:3536982 errors:0 dropped:0 overruns:0 carrier:0
          collisions:0 txqueuelen:1000
          RX bytes:3380060233 (3.1 GiB)  TX bytes:713438255 (680.3 MiB)
          Interrupt:17 Memory:da000000-da012800

lo        Link encap:Local Loopback
          inet addr:127.0.0.1  Mask:255.0.0.0
          inet6 addr: ::1/128 Scope:Host
          UP LOOPBACK RUNNING  MTU:16436  Metric:1
          RX packets:805234 errors:0 dropped:0 overruns:0 frame:0
          TX packets:805234 errors:0 dropped:0 overruns:0 carrier:0
          collisions:0 txqueuelen:0
          RX bytes:132177529 (126.0 MiB)  TX bytes:132177529 (126.0 MiB)
'''
        ifconfig = IfConfig(stdout=fullOld)
        self.assertEqual(len(ifconfig.devices), 2)

        # Verify eth0:
        eth0 = ifconfig.devices[0]
        # eth0      Link encap:Ethernet  HWaddr 00:11:22:33:44:55
        self.assertEqual(eth0.name, 'eth0')
        self.assertEqual(eth0.hwtitle, 'Ethernet')
        self.assertEqual(eth0.hwaddr, '00:11:22:33:44:55')
        # inet addr:1.12.123.124  Bcast:1.12.123.255  Mask:255.255.252.0
        self.assertEqual(eth0.inet, '1.12.123.124')
        self.assertEqual(eth0.netmask, '255.255.252.0')
        self.assertEqual(eth0.broadcast, '1.12.123.255')
        #       inet6 addr: dddd::dddd:dddd:dddd:dddd:dddd:dddd/64 Scope:Site
        # self.assertEqual(eth0.inet6, 'dddd::dddd:dddd:dddd:dddd:dddd:dddd')
        # self.assertEqual(eth0.prefixlen, 64)
        # self.assertEqual(eth0.scopeid, 'Site')
        #       inet6 addr: eeee::eeee:eeee:eeee:eeee:eeee:eeee/64 Scope:Global
        # self.assertEqual(eth0.inet6, 'eeee::eeee:eeee:eeee:eeee:eeee:eeee')
        # self.assertEqual(eth0.prefixlen, 64)
        # self.assertEqual(eth0.scopeid, 'Global')
        #           inet6 addr: ffff::ffff:ffff:ffff:ffff/64 Scope:Link
        self.assertEqual(eth0.inet6, 'ffff::ffff:ffff:ffff:ffff')
        self.assertEqual(eth0.prefixlen, 64)
        self.assertEqual(eth0.scopeid, 'Link')
        #          UP BROADCAST RUNNING MULTICAST  MTU:1500  Metric:1
        self.assertEqual(eth0.flagsstr, 'UP BROADCAST RUNNING MULTICAST')
        self.assertEqual(eth0.mtu, 1500)
        self.assertEqual(eth0.metric, 1)
        #          RX packets:4458926 errors:0 dropped:0 overruns:0 frame:0
        self.assertEqual(eth0.rxpackets, 4458926)
        self.assertEqual(eth0.rxerrors, 0)
        self.assertEqual(eth0.rxdropped, 0)
        self.assertEqual(eth0.rxoverruns, 0)
        self.assertEqual(eth0.rxframe, 0)
        #          TX packets:3536982 errors:0 dropped:0 overruns:0 carrier:0
        self.assertEqual(eth0.txpackets, 3536982)
        self.assertEqual(eth0.txerrors, 0)
        self.assertEqual(eth0.txdropped, 0)
        self.assertEqual(eth0.txoverruns, 0)
        self.assertEqual(eth0.txcarrier, 0)
        #          collisions:0 txqueuelen:1000
        self.assertEqual(eth0.txcollisions, 0)
        self.assertEqual(eth0.txqueuelen, 1000)
        # RX bytes:3380060233 (3.1 GiB)  TX bytes:713438255 (680.3 MiB)
        self.assertEqual(eth0.rxbytes, 3380060233)
        self.assertEqual(eth0.txbytes, 713438255)
        #          Interrupt:17 Memory:da000000-da012800
        self.assertEqual(eth0.interrupt, 17)
        self.assertEqual(eth0.memstart, 0xda000000)
        self.assertEqual(eth0.memend, 0xda012800)

        # Verify lo:
        lo = ifconfig.devices[1]
        # lo        Link encap:Local Loopback
        self.assertEqual(lo.name, 'lo')
        self.assertEqual(lo.hwtitle, 'Local Loopback')
        self.assertEqual(lo.hwaddr, None)
        #          inet addr:127.0.0.1  Mask:255.0.0.0
        self.assertEqual(lo.inet, '127.0.0.1')
        self.assertEqual(lo.netmask, '255.0.0.0')
        self.assertEqual(lo.broadcast, None)
        #          inet6 addr: ::1/128 Scope:Host
        self.assertEqual(lo.inet6, '::1')
        self.assertEqual(lo.prefixlen, 128)
        self.assertEqual(lo.scopeid, 'Host')
        #          UP LOOPBACK RUNNING  MTU:16436  Metric:1
        self.assertEqual(lo.flagsstr, 'UP LOOPBACK RUNNING')
        self.assertEqual(lo.mtu, 16436)
        self.assertEqual(lo.metric, 1)
        #          RX packets:805234 errors:0 dropped:0 overruns:0 frame:0
        self.assertEqual(lo.rxpackets, 805234)
        #          TX packets:805234 errors:0 dropped:0 overruns:0 carrier:0
        self.assertEqual(lo.txpackets, 805234)
        #          collisions:0 txqueuelen:0
        # RX bytes:132177529 (126.0 MiB)  TX bytes:132177529 (126.0 MiB)
        self.assertEqual(lo.rxbytes, 132177529)
        self.assertEqual(lo.txbytes, 132177529)

    def parse_single_device(self, stdout, debug=False):
        ifconfig = IfConfig(stdout, debug)
        self.assertEqual(len(ifconfig.devices), 1)
        return ifconfig.devices[0]

    def test_parsing_flags_line(self):
        dev = self.parse_single_device(
            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500\n')
        self.assertEqual(dev.name, 'foo')
        self.assertEqual(dev.flagsint, 4163)
        self.assertEqual(dev.flagsstr, 'UP,BROADCAST,RUNNING,MULTICAST')
        self.assertEqual(dev.mtu, 1500)

        dev = self.parse_single_device(
            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500  '
            'outfill 32  keepalive 96\n')
        self.assertEqual(dev.mtu, 1500)
        self.assertEqual(dev.outfill, 32)
        self.assertEqual(dev.keepalive, 96)

        # old format
        dev = self.parse_single_device(
            'foo      Link encap:Ethernet  HWaddr F0:F0:F0:F0:F0:F0\n'
            '         UP BROADCAST RUNNING MULTICAST  MTU:500  Metric:2  '
            'Outfill:55  Keepalive:66\n')
        self.assertEqual(dev.flagsstr, 'UP BROADCAST RUNNING MULTICAST')
        self.assertEqual(dev.mtu, 500)
        self.assertEqual(dev.metric, 2)
        self.assertEqual(dev.outfill, 55)
        self.assertEqual(dev.keepalive, 66)

    def test_parsing_ip(self):
        dev = self.parse_single_device(
            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500\n'
            '        inet 1.12.123.124  netmask 255.255.252.0  broadcast '
            '1.12.123.255\n')
        self.assertEqual(dev.inet, '1.12.123.124')
        self.assertEqual(dev.netmask, '255.255.252.0')
        self.assertEqual(dev.get_netmask_bits(), 22)
        self.assertEqual(dev.broadcast, '1.12.123.255')
        self.assertEqual(dev.destination, None)

        dev = self.parse_single_device(
            'lo: flags=73<UP,LOOPBACK,RUNNING>  mtu 16436\n'
            '        inet 127.0.0.1  netmask 255.0.0.0\n')
        self.assertEqual(dev.inet, '127.0.0.1')
        self.assertEqual(dev.netmask, '255.0.0.0')
        self.assertEqual(dev.get_netmask_bits(), 8)

        self.assertEqual(dev.broadcast, None)
        self.assertEqual(dev.destination, None)

        dev = self.parse_single_device(
            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500\n'
            '        inet 1.1.1.1  netmask 255.255.255.254  destination '
            '1.12.123.255\n')
        self.assertEqual(dev.inet, '1.1.1.1')
        self.assertEqual(dev.netmask, '255.255.255.254')
        self.assertEqual(dev.broadcast, None)
        self.assertEqual(dev.destination, '1.12.123.255')

        # old format
        dev = self.parse_single_device(
            'foo      Link encap:Ethernet  HWaddr F0:F0:F0:F0:F0:F0\n'
            '       inet addr:1.2.3.4  P-t-P:10.20.30.40  '
            'Mask:255.255.254.0\n')
        self.assertEqual(dev.inet, '1.2.3.4')
        self.assertEqual(dev.destination, '10.20.30.40')
        self.assertEqual(dev.broadcast, None)
        self.assertEqual(dev.get_netmask_bits(), 23)
        self.assertEqual(dev.netmask, '255.255.254.0')

    def test_parsing_device_line(self):
        dev = self.parse_single_device(
            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500\n'
            '        device interrupt 20  memory 0xf2600000-f2620000  \n')
        self.assertEqual(dev.interrupt, 20)
        self.assertEqual(dev.baseaddr, None)
        self.assertEqual(dev.memstart, 0xf2600000)
        self.assertEqual(dev.memend, 0xf2620000)
        self.assertEqual(dev.dma, None)

        # but the fields in the "device" field are optional:
        dev = self.parse_single_device(
            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500\n'
            '        device \n')
        self.assertEqual(dev.interrupt, None)
        self.assertEqual(dev.baseaddr, None)
        self.assertEqual(dev.memstart, None)
        self.assertEqual(dev.memend, None)
        self.assertEqual(dev.dma, None)

        dev = self.parse_single_device(
            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500\n'
            '        device memory 0xf2600000-f2620000  \n')
        self.assertEqual(dev.interrupt, None)
        self.assertEqual(dev.baseaddr, None)
        self.assertEqual(dev.memstart, 0xf2600000)
        self.assertEqual(dev.memend, 0xf2620000)
        self.assertEqual(dev.dma, None)

        # and there could potentially be "base" and "dma" fields:
        dev = self.parse_single_device(
            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500\n'
            '        device base 0xdeadbeef  \n')
        self.assertEqual(dev.interrupt, None)
        self.assertEqual(dev.baseaddr, 0xdeadbeef)
        self.assertEqual(dev.memstart, None)
        self.assertEqual(dev.memend, None)
        self.assertEqual(dev.dma, None)

        dev = self.parse_single_device(
            'foo: flags=4163<UP,BROADCAST,RUNNING,MULTICAST>  mtu 1500\n'
            '        device interrupt 20  dma 0xbad4f00d\n')
        self.assertEqual(dev.interrupt, 20)
        self.assertEqual(dev.baseaddr, None)
        self.assertEqual(dev.memstart, None)
        self.assertEqual(dev.memend, None)
        self.assertEqual(dev.dma, 0xbad4f00d)

        # old format
        dev = self.parse_single_device(
            'foo      Link encap:Ethernet  HWaddr F0:F0:F0:F0:F0:F0\n'
            '         Interrupt:20 Base address:0xdeadbeef DMA chan:bad4f00d\n'
            '         collisions:13 compressed:11 \n')
        self.assertEqual(dev.interrupt, 20)
        self.assertEqual(dev.baseaddr, 0xdeadbeef)
        self.assertEqual(dev.dma, 0xbad4f00d)
        self.assertEqual(dev.txcollisions, 13)
        self.assertEqual(dev.txcompressed, 11)

    def test_parse_ip4addr(self):
        self.assertEqual(parse_ip4addr('1.1.1.1'), b'\x01\x01\x01\x01')
        self.assertEqual(parse_ip4addr('127.0.0.1'), b'\x7f\x00\x00\x01')

    def test_local(self):
        # Actually invoke ifconfig locally, and parse whatever it emits:
        IfConfig()


if __name__ == '__main__':
    unittest.main()



# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
#   Copyright (c) 2011, 2012, 2013  Red Hat, Inc. All rights reserved.
#
#   This copyrighted material is made available to anyone wishing
#   to use, modify, copy, or redistribute it subject to the terms
#   and conditions of the GNU General Public License version 2.
#
#   This program is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#   PURPOSE. See the GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the Free
#   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#   Boston, MA 02110-1301, USA.
#
#   Author: Dave Malcolm <dmalcolm@redhat.com>
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

import unittest

import ethtool

from .parse_ifconfig import IfConfig

INVALID_DEVICE_NAME = "I am not a valid device name"

# Screenscrape the output from "ifconfig".  We will use this below to validate
# the results from the "ethtool" module:
ifconfig = IfConfig()
for dev in ifconfig.devices:
    print(dev)


class EthtoolTests(unittest.TestCase):

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #  asserts
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    def assertIsString(self, value):
        self.assertTrue(isinstance(value, str))

    def assertIsStringOrNone(self, value):
        if value is not None:
            self.assertTrue(isinstance(value, str))

    def assertIsInt(self, value):
        self.assertTrue(isinstance(value, int))

    def assertRaisesIOError(self, fn, args, errmsg):
        """
        Verify that an IOError is raised, and that the errno and message are
        as expected

        (Python 2.6 and earlier's assertRaises doesn't allow us to check the
        details of the exception that is raised)
        """
        try:
            fn(*args)
        except IOError as e:
            # Check the details of the exception:
            enum, emsg = e.args
            self.assertEqual('[Errno {}] {}'.format(enum, emsg), errmsg)
        else:
            self.fail('IOError was not raised calling %s on %s' % (fn, args))

    def assertRaisesNoSuchDevice(self, fn, *args):
        self.assertRaisesIOError(fn, args, '[Errno 19] No such device')

    def assertIsStringExceptForLoopback(self, fn, devname, errmsg):
        if devname == 'lo':
            self.assertRaisesIOError(fn, (devname, ), errmsg)
        else:
            self.assertIsString(fn(devname))

    def assertEqualIpv4Str(self, ethtooladdr, scrapedaddr):
        if scrapedaddr is None:
            self.assertEqual(ethtooladdr, '0.0.0.0')
        else:
            self.assertEqual(ethtooladdr, scrapedaddr)

    def assertEqualHwAddr(self, ethtooladdr, scrapedaddr):
        if scrapedaddr is None:
            self.assertEqual(ethtooladdr, '00:00:00:00:00:00')
        else:
            self.assertEqual(ethtooladdr, scrapedaddr.lower())

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #  helpers
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    def _functions_accepting_devnames(self, devname):
        self.assertIsString(devname)

        scraped = ifconfig.get_device_by_name(devname)

        try:
            broadcast = ethtool.get_broadcast(devname)
        except (OSError, IOError):
            # Some devices might end up with
            # [Errno 99] Cannot assign requested address
            # That's IOError on 2.7, OSError on 3
            assignable = False
        else:
            assignable = True
            self.assertIsString(broadcast)

            # Broadcast is optional in ifconfig output
            if scraped.broadcast:
                self.assertEqualIpv4Str(broadcast, scraped.broadcast)

        self.assertIsStringExceptForLoopback(ethtool.get_businfo, devname,
                                             '[Errno 95] Operation not '
                                             'supported')

        self.assertIsInt(ethtool.get_flags(devname))
        # flagsint cannot be obtained from old ifconfig format
        if not ifconfig.oldFormat:
            self.assertEqual(ethtool.get_flags(devname), scraped.flagsint)
        self.assertIsInt(ethtool.get_gso(devname))
        self.assertIsInt(ethtool.get_gro(devname))
        self.assertIsString(ethtool.get_hwaddr(devname))
        self.assertEqualHwAddr(ethtool.get_hwaddr(devname),
                               scraped.hwaddr)

        if assignable:
            self.assertIsString(ethtool.get_ipaddr(devname))
            self.assertEqual(ethtool.get_ipaddr(devname), scraped.inet)

        self.assertIsStringExceptForLoopback(ethtool.get_module, devname,
                                             '[Errno 95] Operation not '
                                             'supported')

        if assignable:
            self.assertIsString(ethtool.get_netmask(devname))
            self.assertEqual(ethtool.get_netmask(devname),
                             scraped.netmask)

        # Operation is not supported only on loopback device
        if devname == 'lo':
            self.assertRaisesIOError(ethtool.get_ringparam, (devname, ),
                                     '[Errno 95] Operation not supported')

        self.assertIsInt(ethtool.get_sg(devname))

        try:
            self.assertIsInt(ethtool.get_ufo(devname))
        except (OSError, IOError):
            # This test may fail due to insufficient privileges
            # That's IOError on 2.7, OSError (PermissionError) on 3
            pass

        self.assertIsInt(ethtool.get_tso(devname))

        # TODO: self.assertIsString(ethtool.set_coalesce(devname))

        # TODO: self.assertIsString(ethtool.set_ringparam(devname))

        # TODO: self.assertIsString(ethtool.set_tso(devname))

        # TODO: self.assertIsString(ethtool.set_gso(devname))

        # TODO: self.assertIsString(ethtool.set_gro(devname))

    def _verify_etherinfo_object(self, ei):
        self.assertTrue(isinstance(ei, ethtool.etherinfo))
        self.assertIsString(ei.device)

        try:
            scraped = ifconfig.get_device_by_name(ei.device)
        except ValueError:
            scraped = None

        self.assertIsStringOrNone(ei.ipv4_address)
        if scraped:
            self.assertEqual(ei.ipv4_address, scraped.inet)

        self.assertIsStringOrNone(ei.ipv4_broadcast)
        if scraped and scraped.broadcast:
            # Broadcast is optional
            self.assertEqual(ei.ipv4_broadcast, scraped.broadcast)

        self.assertIsInt(ei.ipv4_netmask)
        if scraped:
            self.assertEqual(ei.ipv4_netmask, scraped.get_netmask_bits())

        self.assertIsStringOrNone(ei.mac_address)
        if scraped and scraped.hwaddr and scraped.hwtitle.lower() != 'unspec':
            scraped.hwaddr = scraped.hwaddr.lower()
            self.assertEqualHwAddr(ei.mac_address.lower(), scraped.hwaddr)

        i6s = ei.get_ipv6_addresses()
        for i6 in i6s:
            self.assertTrue(isinstance(i6, ethtool.NetlinkIPaddress))
            self.assertIsString(i6.address)
            self.assertIsInt(i6.netmask)
            self.assertIsString(i6.scope)

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #  tests
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    def test_invalid_devices(self):
        # Verify sane handling of non-existant devices

        get_fns = ('get_broadcast', 'get_businfo', 'get_coalesce', 'get_flags',
                   'get_gso', 'get_gso', 'get_hwaddr', 'get_ipaddr',
                   'get_module', 'get_netmask', 'get_ringparam', 'get_sg',
                   'get_tso', 'get_ufo')
        for fnname in get_fns:
            self.assertRaisesNoSuchDevice(getattr(ethtool, fnname),
                                          INVALID_DEVICE_NAME)

        set_fns = ('set_coalesce', 'set_ringparam', 'set_tso', 'set_gso',
                   'set_gro')
        for fnname in set_fns:
            # Currently this fails, with an IOError from
            #   ethtool.c:__struct_desc_from_dict
            # with message:
            #   'Missing dict entry for field rx_coalesce_usecs'
            if False:
                self.assertRaisesNoSuchDevice(getattr(ethtool, fnname),
                                              INVALID_DEVICE_NAME, 42)

    def test_get_interface_info_invalid(self):
        eis = ethtool.get_interfaces_info(INVALID_DEVICE_NAME)
        self.assertEqual(len(eis), 1)
        ei = eis[0]
        self.assertEqual(ei.device, INVALID_DEVICE_NAME)
        self.assertRaisesIOError(
            getattr, (ei, 'ipv4_address'), '[Errno 19] No such device')
        self.assertRaisesIOError(
            getattr, (ei, 'ipv4_netmask'), '[Errno 19] No such device')
        self.assertRaisesIOError(
            getattr, (ei, 'ipv4_broadcast'), '[Errno 19] No such device')
        self.assertRaisesIOError(
            getattr, (ei, 'mac_address'), '[Errno 19] No such device')

    def test_get_interface_info_active(self):
        eis = ethtool.get_interfaces_info(ethtool.get_active_devices())
        for ei in eis:
            self._verify_etherinfo_object(ei)

    def test_get_interface_info_all(self):
        eis = ethtool.get_interfaces_info(ethtool.get_devices())
        for ei in eis:
            self._verify_etherinfo_object(ei)

    def test_get_active_devices(self):
        for devname in ethtool.get_active_devices():
            # Skip these test on tun devices
            if devname.startswith('tun'):
                continue
            self._functions_accepting_devnames(devname)

    def test_etherinfo_objects(self):
        devnames = ethtool.get_devices()
        eis = ethtool.get_interfaces_info(devnames)
        for ei in eis:
            self._verify_etherinfo_object(ei)


if __name__ == '__main__':
    unittest.main()



import unittest
import ethtool
import sys
import os
from io import TextIOWrapper, BytesIO
from imp import load_source

pifc = load_source('pifc', 'scripts/pifconfig')
peth = load_source('peth', 'scripts/pethtool')


def find_suitable_device():
    active_devices = ethtool.get_active_devices()
    for device in active_devices:
        device_flags = ethtool.get_flags(device)
        str_flags = pifc.flags2str(device_flags)
        try:
            wireless_protocol = ethtool.get_wireless_protocol(device)
        except Exception:
            wireless_protocol = None

        if 'UP' in str_flags and 'RUNNING' in str_flags \
                and 'BROADCAST' in str_flags and not wireless_protocol:
            return device

    return None


loopback = 'lo'
device = find_suitable_device()


class ScriptsTests(unittest.TestCase):

    def setUp(self):
        self._old_stdout = sys.stdout
        self._stdout = sys.stdout = TextIOWrapper(BytesIO(), sys.stdout.encoding)

    def _output(self):
        self._stdout.seek(0)
        return self._stdout.read()

    def tearDown(self):
        sys.stdout = self._old_stdout
        self._stdout.close()

    def test_flags2str(self):
        self.assertEqual(pifc.flags2str(0), '')
        self.assertEqual(pifc.flags2str(73), 'UP LOOPBACK RUNNING')
        self.assertEqual(pifc.flags2str(4305), 'UP POINTOPOINT RUNNING NOARP MULTICAST')
        self.assertEqual(pifc.flags2str(4163), 'UP BROADCAST RUNNING MULTICAST')

    # Tests for loopback

    def test_driver_lo(self):
        self.assertIsNone(peth.show_driver(loopback))
        self.assertEqual(self._output(),
                         'driver: not implemented\nbus-info: not available\n'
                         )

    def test_show_ring_lo(self):
        self.assertIsNone(peth.show_ring(loopback))
        self.assertEqual(self._output(),
                         'Ring parameters for {}:\n  NOT supported!\n'.format(loopback)
                         )

    def test_show_coalesce_lo(self):
        self.assertIsNone(peth.show_coalesce(loopback))
        self.assertEqual(self._output(),
                         'Coalesce parameters for {}:\n  NOT supported!\n'.format(loopback)
                         )

    def test_show_offload_lo(self):
        self.assertIsNone(peth.show_offload(loopback))

        # Check if we have rights to obtain ufo and set proper expected output
        try:
            ethtool.get_ufo(loopback)
            expected_ufo = 'on'
        except (OSError, IOError):
            expected_ufo = 'not supported'

        self.assertEqual(self._output(),
                         '''scatter-gather: on
tcp segmentation offload: on
udp fragmentation offload: {expected_ufo}
generic segmentation offload: on
generic receive offload: on
'''.format(expected_ufo=expected_ufo)
                         )

    # Tests for another device
    if device:
        def test_driver_eth(self):
            self.assertIsNone(peth.show_driver(device))
            expected_lines_start = ['driver: ', 'bus-info: ']
            lines = self._output().split('\n')
            for expected_start, line in zip(expected_lines_start, lines):
                self.assertTrue(line.startswith(expected_start))

        def test_show_ring_eth(self):
            self.assertIsNone(peth.show_ring(device))
            expected_lines_start = ['Ring parameters for ',
                                    'Pre-set maximums:', 'RX:', 'RX Mini:',
                                    'RX Jumbo:', 'TX', 'Current hardware settings:',
                                    'RX:', 'RX Mini:', 'RX Jumbo:', 'TX:']
            lines = self._output().split('\n')
            for expected_start, line in zip(expected_lines_start, lines):
                self.assertTrue(line.startswith(expected_start))

        @unittest.skipIf('TRAVIS' in os.environ and os.environ['TRAVIS'] == 'true',
                         'Skipping this test on Travis CI because show '
                         'coalesce is not supported on ethernet device in VM.')
        def test_show_coalesce_eth(self):
            self.assertIsNone(peth.show_coalesce(device))
            expected_lines_start = ['Coalesce parameters for',
                                    'Adaptive RX:',
                                    'stats-block-usecs:',
                                    'sample-interval:',
                                    'pkt-rate-low:',
                                    'pkt-rate-high:',
                                    'rx-usecs:',
                                    'rx-frames:',
                                    'rx-usecs-irq:',
                                    'rx-frames-irq:',
                                    'tx-usecs:',
                                    'tx-frames:',
                                    'tx-usecs-irq:',
                                    'tx-frames-irq:',
                                    'rx-usecs-low:',
                                    'rx-frame-low:',
                                    'tx-usecs-low:',
                                    'tx-frame-low:',
                                    'rx-usecs-high:',
                                    'rx-frame-high:',
                                    'tx-usecs-high:',
                                    'tx-frame-high:']

            lines = self._output().split('\n')
            for expected_start, line in zip(expected_lines_start, lines):
                self.assertTrue(line.startswith(expected_start))

        def test_show_offload_eth(self):
            self.assertIsNone(peth.show_offload(device))
            expected_lines_start = ['scatter-gather:',
                                    'tcp segmentation offload:',
                                    'udp fragmentation offload:',
                                    'generic segmentation offload:']
            lines = self._output().split('\n')
            for expected_start, line in zip(expected_lines_start, lines):
                self.assertTrue(line.startswith(expected_start))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function

from setuptools import setup, Extension
import sys
from io import open

try:
    import commands
except ImportError:
    import subprocess as commands

version = '0.14'


class PkgConfigExtension(Extension):
    '''Extension with lazy properties taken from pkg-config'''

    def __init__(self, *args, **kwargs):
        '''Behaves like Extension's __init__ but you should not use
        include_dirs, library_dirs and libraries arguments.

        Extra arguments:
          pkg : string
            The name of the pkg-config package to use for querying
          extra_libraris : [string]
            This will be added to the libraries attribute. Optional.
        '''
        self._pkg = kwargs['pkg']
        del kwargs['pkg']
        if 'extra_libraries' in kwargs:
            self._extra_libraries = kwargs['extra_libraries']
            del kwargs['extra_libraries']
        else:
            self._extra_libraries = []

        Extension.__init__(self, *args, **kwargs)

        try:
            # on Python 2 we need to delete those now
            del self.include_dirs
            del self.library_dirs
            del self.libraries
        except AttributeError:
            # on Python 3, that's not needed or possible
            pass

    @classmethod
    def _str2list(cls, pkgstr, onlystr):
        res = []
        for l in pkgstr.split(" "):
            if l.find(onlystr) == 0:
                res.append(l.replace(onlystr, "", 1))
        return res

    @classmethod
    def _run(cls, command_string):
        res, output = commands.getstatusoutput(command_string)
        if res != 0:
            print('Failed to query %s' % command_string)
            sys.exit(1)
        return output

    @property
    def include_dirs(self):
        includes = self._run('pkg-config --cflags-only-I %s' % self._pkg)
        return self._str2list(includes, '-I')

    @property
    def library_dirs(self):
        libdirs = self._run('pkg-config --libs-only-L %s' % self._pkg)
        return self._str2list(libdirs, '-L')

    @property
    def libraries(self):
        libs = self._run('pkg-config --libs-only-l %s' % self._pkg)
        return self._str2list(libs, '-l') + self._extra_libraries

    @include_dirs.setter
    def include_dirs(self, value):
        pass

    @library_dirs.setter
    def library_dirs(self, value):
        pass

    @libraries.setter
    def libraries(self, value):
        pass


with open('README.rst', encoding='utf-8') as f:
    long_description = f.read()

with open('CHANGES.rst', encoding='utf-8') as f:
    long_description += '\n\n'
    long_description += f.read()

setup(name='ethtool',
      version=version,
      description='Python module to interface with ethtool',
      long_description=long_description,

      author='Harald Hoyer, Arnaldo Carvalho de Melo, David Sommerseth',
      author_email='davids@redhat.com',

      maintainer='Lumír Balhar, Miro Hrončok, Charalampos Stratakis, Sanqui',
      maintainer_email='python-maint@redhat.com',

      url='https://github.com/fedora-python/python-ethtool',
      license='GPL-2.0',
      keywords='network networking ethernet tool ethtool',

      classifiers=[
          'Development Status :: 7 - Inactive',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'Operating System :: POSIX :: Linux',
          'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: Implementation :: CPython',
          'Topic :: Software Development :: Libraries',
          'Topic :: System :: Networking',
      ],
      
      scripts=[
        'scripts/pethtool', 'scripts/pifconfig'
      ],
      
      ext_modules=[
          PkgConfigExtension(
              'ethtool',
              sources=[
                  'python-ethtool/ethtool.c',
                  'python-ethtool/etherinfo.c',
                  'python-ethtool/etherinfo_obj.c',
                  'python-ethtool/netlink.c',
                  'python-ethtool/netlink-address.c'],
              extra_compile_args=[
                  '-fno-strict-aliasing', '-Wno-unused-function'],
              define_macros=[('VERSION', '"%s"' % version)],
              pkg='libnl-3.0',
              extra_libraries=['nl-route-3'],
          )
      ]
      )
